/*
* Copyright Milan Technology Inc., 1991, 1992
*/

/* @(#)std.h	2.1 11/10/93 */

#define _args( x ) ( ) 

typedef char bool;
typedef char i8;
typedef unsigned char u8;
typedef short i16;
typedef unsigned short u16;
typedef long i32;
typedef unsigned long u32;

